# Example 6.6 of
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo
###############################################################################

# Use only data after calendar year 1982
library(lmfor)
library(lme4)

data(patti)
# use only data of year 1982 ->
patti<-patti[patti$Year>1982,]

# Transformation for natural spline regression
# Eq 2.25 in Harrells book, p. 20
# t is a vector of knots, j the component to be calculated
natspline<-function(X,t,j){
                     k<-length(t)
                     pmax(0,X-t[j])^3-
                     pmax(0,X-t[k-1])^3*(t[k]-t[j])/(t[k]-t[k-1])+
                     pmax(0,X-t[k])^3*(t[k-1]-t[j])/(t[k]-t[k-1])
}

# Use 3 knots to model the age trend using a regression spline.
# Compute the required spline component.
te<-quantile(patti$CA,probs=c(0.1,0.5,0.9)) 
patti$e1<-natspline(X=patti$CA,t=te,j=1)

# Remove the observations of thinned plots after thinning
patti2<-patti[patti$SDClass==1|patti$Year<1987,]

extmod<-lmer(log(RBA)~CA+e1+(1|Tree)+(1|Year),data=patti2)

patti$logpred<-predict(extmod,newdata=patti,
                       re.form=~(1|Tree)+(1|Year))

# Back-transformation with the two-point bias correction
patti$trend<-(exp(patti$logpred+summary(extmod)$sigma)+
              exp(patti$logpred-summary(extmod)$sigma))/2

# Compute the thinning effects to the dataset.
patti$ThEf<-patti$RBA-patti$trend

windows(width=7,height=3)
par(mfcol=c(1,2),cex=0.9,mai=c(0.6,0.6,0.1,0.1), mgp=c(2,0.7,0))

linesplot(patti$Year,patti$RBA,patti$Tree,
		  col.lin=gray(0.7),cex=0,
		  ylab="RBA",xlab="Year",lty="dashed")
linesplot(patti2$Year,patti2$RBA,patti2$Tree,
		  col.lin=gray(1-(as.numeric(afterthin$SDClass)+1)/6),
#		  lwd=as.numeric(afterthin$SDClass)/2+0.5,
		  cex=0,add=TRUE)
for (i in 1:4) {
	thisclass<-patti[patti$SDClass==i,]
	lsmooth<-loess(RBA~Year,data=thisclass,span=0.5)
	apu<-data.frame(Year=seq(min(thisclass$Year),max(thisclass$Year),length=30))
	fit<-predict(lsmooth, apu)
	lines(apu$Year,fit,lwd=5,col=gray(1-(i+1)/6))
}

linesplot(patti$Year,patti$ThEf,patti$Tree,
		  col.lin=gray(0.7),cex=0,
		  ylab="Thinning Effect",xlab="Year",lty="dashed")
patti2<-patti[patti$SDClass==1|patti$Year<1987,]
linesplot(patti2$Year,patti2$ThEf,patti2$Tree,
		col.lin=gray(1-(as.numeric(afterthin$SDClass)+1)/6),
		cex=0,lwd=2,add=TRUE)
for (i in 1:4) {
	thisclass<-patti[patti$SDClass==i,]
	lsmooth<-loess(ThEf~Year,data=thisclass,span=0.5)
	apu<-data.frame(Year=seq(min(thisclass$Year),max(thisclass$Year),length=30))
	fit<-predict(lsmooth, apu)
	lines(apu$Year,fit,lwd=5,col=gray(1-(i+1)/6))
}
### 
